
"""
To install the python bindings (on a mac)

    > make python
    > pip install -e .

"""


import pytest
import ripser


def test_import():
    import ripser

def test_instantiate():
    rip = ripser.Rips()

    assert rip is not None


import subprocess
import os

import time

import matplotlib.pyplot as plt

import numpy as np
from sklearn.base import BaseEstimator
from sklearn.metrics.pairwise import pairwise_distances

from pyRipser import doRipsFiltrationDM as DRFDM


class Rips(BaseEstimator):
    """Wrapper around Uli Bauer's Ripser code 

    Example usage:

    ```
        from ripser import Rips
        from sklearn import datasets

        data = datasets.make_circles(n_samples=110)[0]
        rips = Rips()
        rips.transform(data)
        rips.plot()
    ```

    """

    def __init__(self, maxdim=1, thresh=-1, coeff=2):
        self.maxdim = maxdim
        self.thresh = thresh
        self.coeff = coeff

    def transform(self, X, distance_matrix=False, metric='euclidean'):

        if not distance_matrix:
            X = pairwise_distances(X, metric=metric)

        dgm = self.compute_rips(X)
        self._dgm = dgm
    
    def fit_transform(self, X, distance_matrix=False, metric='euclidean'):
        """ Run transform and return results

        """
        self.transform(X, distance_matrix, metric)
        return self._dgm

    def compute_rips(self, dm):
        """ Compute the persistence diagram
       
        :param D: An NxN pairwise distance matrix
        :param maxHomDim: The dimension up to which to compute persistent homology
        :param thresh: Threshold up to which to add edges.  If not specified, add all
            edges up to the full clique
        :param coeff: A prime to use as the field coefficients for the PH computation

        :return: PDs (array of all persistence diagrams from 0D up to maxHomDim).
            Each persistence diagram is a numpy array
        """
        
        N = dm.shape[0]
        if self.thresh == -1:
            thresh = np.max(dm)*2
        [I, J] = np.meshgrid(np.arange(N), np.arange(N))
        DParam = np.array(dm[I > J], dtype=np.float32)

        res = DRFDM(DParam, self.maxdim, thresh, self.coeff)
        PDs = []
        istart = 0
        for dim in range(self.maxdim + 1):
            N = int(res[-1 - (self.maxdim - dim)])
            if dim > 0:
                N -= int(res[-2 - (self.maxdim - dim)])
            I = np.array(res[istart * 2:(istart + N) * 2])
            PDs.append(np.reshape(I, (N, 2)))
            istart += N
        return PDs

    def plot(self, diagram=None, diagonal=True, sz=20, label='dgm', axcolor=np.array([0.0, 0.0, 0.0]), marker=None, show=True):
        """ Plot each diagram on the same plot.
        """
        if diagram is None:
            diagram = self._dgm
        
        if type(diagram) is not list:
            diagram = [diagram]
    
        colors = ['r','g', 'b'] # TODO: convert this to a cylic generator so we can zip as many as required.
        for dgm, color in zip(diagram, colors):
            
            if dgm.size is not 0:
                # build diagonal line

                if diagonal:
                    axMin, axMax = np.min(dgm), np.max(dgm)
                    axRange = axMax - axMin

                    a = max(axMin - axRange / 5, 0)
                    b = axMax + axRange / 5

                    # plot diagonal line
                    plt.plot([a, b], [a, b], '--', c=axcolor, label='none')
                
                # plot points
                plt.scatter(dgm[:, 0], dgm[:, 1], sz, 
                                color, label=label, edgecolor='none')
                # add labels
                plt.xlabel('Birth')
                plt.ylabel('Death')
                # plt.axis('equal')

        if show:
            plt.show()






import sys
from distutils.core import setup, Extension

# we'd better have Cython installed, or it's a no-go
try:
	from Cython.Build import cythonize
	from Cython.Distutils import build_ext
except:
    print("You don't seem to have Cython installed. Please get a")
    print("copy from www.cython.org or install it with `pip install Cython`")
    sys.exit(1)


with open('README.md') as f:
    long_description = f.read()

#c++ -std=c++11 ripser.cpp -o ripser -Ofast -D NDEBUG -D PRINT_PERSISTENCE_PAIRS
#options = ["-std=c++11", "-Ofast"]
options = ["-std=c++11", "-Ofast"]

if sys.version_info[0] == 2:
	options.append("-fpermissive")

class CustomBuildExtCommand(build_ext):
	""" This extension command lets us not require numpy be installed before running pip install ripser """
	"""build_ext command for use when numpy headers are needed."""
	def run(self):
		# Import numpy here, only when headers are needed
		import numpy
		# Add numpy headers to include_dirs
		self.include_dirs.append(numpy.get_include())
		# Call original build_ext command
		build_ext.run(self)





setup(
	name="ripser",
	version='0.1.4',	
	description="Persistent homology for people",
	long_description=long_description,
	long_description_content_type="text/markdown",
	author="Chris Tralie, Nathaniel Saul",
	author_email="chris.tralie@gmail.com, nathaniel.saul@wsu.edu",
	url="https://github.com/sauln/ripser",
	license='LGPL',
	packages=['ripser'],

	ext_modules = cythonize(
	Extension("pyRipser",
		sources = ["ripser/pyRipser.pyx"],
		define_macros=[("USE_COEFFICIENTS", 1), ("PYTHON_EXTENSION", 1), ("NDEBUG", 1), ("ASSEMBLE_REDUCTION_MATRIX", 1)],
		extra_compile_args = options,
		language="c++"
	)),

	install_requires=[
		'Cython',
		'numpy',
		'scipy',
        'matplotlib',
		'scikit-learn'
    ],
	cmdclass = {'build_ext': CustomBuildExtCommand},


)



# distutils: language = c++

cimport numpy as np
cimport ripser.pyRips as pyRips
import cython

@cython.boundscheck(False)
@cython.wraparound(False)
def doRipsFiltrationDM(np.ndarray[float,ndim=1,mode="c"] DParam not None, int maxHomDim, int thresh=-1, int coeff=2):

	cdef int N = DParam.shape[0]

	res = pyRips.pythondm(&DParam[0], N, coeff, maxHomDim, thresh)

	return res
